/******************************************************************************
;
;  Codi d'Exemple per la utilitzaci del LCD
;
;******************************************************************************/

#include <kd_sfrs.h>
#include <lcd.h>


/******************************************************************************
;
;  Programa Principal
;
;******************************************************************************/


/******************************************************************************

  Una manera de calcular el numero de ticks (pulsos) per segon, seria comptar 
  durant un interval de temps definit, quants de ticks s'han rebut.
  
  Per  fer-ho  tendriem  que progrmar  el timer de  manera que  provoques una 
  interrupcio en  un interval que  ens fos fcil arrivar a calcular un segons.
  I programar l'HSI, de manera que poguem comptar totes els tics que rebem.
  
  Una altra possibilitat es comptar el temps que hi ha entre tics.

;******************************************************************************/


void escriu_valor_lcd(unsigned int wValor, int x, int y) {
      
	  unsigned char xifra;
	  unsigned int valor;
	  int i=0;
	  valor=wValor;
	  while (valor > 0) {
		GOTOXY(x-i,y);
		xifra=valor%10+'0';
		valor=valor/10;
		DADA_LCD(xifra);
		i++;
	  }
	  while (i<5) {
		GOTOXY(x-i,y);
		DADA_LCD(' ');
		i++;
	  }
}

void escriu_car_lcd(unsigned int wValor, int x, int y) {

		GOTOXY(x,y);
		DADA_LCD(wValor);
}

unsigned int auxiliar;
unsigned int tics;
unsigned int conta_ovf;

unsigned int total;


void main()
{
  unsigned char bData;  
  
  INICIALITZA_LCD();
  CLRSCR_LCD();
           

////////////////////////////////////////////////////
//               Inicialitzacions                 //
////////////////////////////////////////////////////

	/* Deshabilitem les interrupcions i les PTS */
	asm { 
		dpts;
		di; 
	}
	
	wsr=0x00;
	int_mask |= 0x04;	// HSIData Available, cal configurar IOC1.7 
	int_mask1 |= 0x10;	// T2OVF
	
	wsr=0x0f;
	auxiliar=ioc0;					
	
	wsr=0x00;
	auxiliar |= 0x01;
	ioc0=auxiliar;		// activo hsi0
	auxiliar &=	0xf7;
	ioc0=auxiliar;		// desactivo EXTRST del timer2

	wsr=0x0f;
	auxiliar=ioc1;
	
	wsr=0x00;
	auxiliar &= 0x7f;				
	ioc1=auxiliar;		// La Hsi Data Available (HSIINT) es generada quan 
						// es carrega el Holding Register

	wsr=0x0f;
	auxiliar=ioc2;
	wsr=0x00;
	auxiliar |= 0x02;
	ioc2=auxiliar;		// Configurem el TIMER2 per comptar UP en normal mode, al rang 
						// FFFFh-0000h.
	
	wsr=0x01;
	ioc3 = ioc3 | 0x01;	// Seleccionem el clock intern pel TIMER2
	
	wsr=0x00; 
	timer2=0;			// Carreguem els valors inicials als comptadors
	
	wsr=0x0f;
	auxiliar=hsi_mode;

	wsr=0x00;
	auxiliar |= 0x01;	// cada transici positiva HSI.0 provocar interrupci ...
	auxiliar &= 0xfd;	// ... pero esperem 8 transicions positives (clocks)
	hsi_mode = auxiliar;		
	
/*  Inicialitzem les variables */

	tics=0;
	conta_ovf=0;
	total=0; // Usada para hacer el promedio
       
/*  Finalment cal habilitar les interrupcions per poder servir-les. */  
	asm { ei;}


///////////////////////////////////////////////////////////
//                 Programa principal                    //
//          Ja s'han fet les inicialitzacions            //
///////////////////////////////////////////////////////////

 CLRSCR_LCD();
	
 while(1)
 	{  

// En general:

//   1 mostra = N overflows
//  x tics    1 mostra    1 volta      q m      1 km    3600 s   
// -------- * --------- * -------- * ------- * ------ * ------ = ?
// 1 mostra     z seg      p tics    1 volta   1000 m     1h      


// En este caso:

//   1 mostra = 1 overflows
//  x tics    ops mostra    1 volta    perim m    1 km    3600 s    km
// -------- * ---------- * -------- * ------- * ------ * ------ = ----
// 1 mostra     1 seg      tpv tics   1 volta   1000 m     1h       h

// Nmero de overflows por segundo
#define ops 20

// Nmero de tics que se generan en cada vuelta de la rueda
#define tpv 1

// Permetro de la rueda, en metros. 2*pi*radio
#define perim 2



//#define k ( (ops*perim*36)/(tpv*10) )
// Sustiyendo por los valores que aplicaremos, queda:
#define k 144

// Sin factor
// #define k 1


	// En cada OVF, mirar cuntos tics nos han llegado
	// 'tics' se puso a 0 en la visualizacin anterior

	// Actualizar la pantalla cada 5 overflows
     if(conta_ovf>5){ 

	    // 'tics' tiene el nmero de tics por 1 overflow
	    // Aplicar factor de conversin k para mostrarlo en otra escala
	    	
	  	conta_ovf=0;

	        // escriu_valor_lcd(k*tics,8,1);
		escriu_valor_lcd(k*(total/5),8,1);

	        //tics=0;
	        total=0;


	  }			


     }	  
        
}


void RSI_Hsi() {
	// Comptar els numero de tics que arriven
  	asm {di;}
	tics++;					//contamos tics q van llegando

	// Leer hsi_time para cargar el siguiente valor
	wsr=0x0 ;
	auxiliar=hsi_time;

	asm {ei;}
}

void RSITimer2() {
	// Incrementa el comptador de interrupcions del timer i avisa que sa produit una interrupcio
	// i reprograma el timer2
	asm {di;}
	
	total+=tics;
	tics=0;

	conta_ovf++;			//contamos overflows de tiempo, por si las HSI "tardan"

	// El timer2 sigue contando, automticamente
		
	asm {ei;}
}
